from collections import OrderedDict

from red_connector_ftp.commons.cli_modes import cli_modes
from red_connector_ftp.version import VERSION

from red_connector_ftp.ftp.send_receive_file import receive_file, receive_file_validate
from red_connector_ftp.ftp.send_receive_file import RECEIVE_FILE_DESCRIPTION, RECEIVE_FILE_VALIDATE_DESCRIPTION
# from red_connector_ftp.send_receive_file import send_file, send_file_validate
# from red_connector_ftp.send_receive_file import SEND_FILE_DESCRIPTION, SEND_FILE_VALIDATE_DESCRIPTION

# from red_connector_ftp.send_receive_dir import receive_dir, receive_dir_validate
# from red_connector_ftp.send_receive_dir import RECEIVE_DIR_DESCRIPTION, RECEIVE_DIR_VALIDATE_DESCRIPTION
# from red_connector_ftp.send_receive_dir import send_dir, send_dir_validate
# from red_connector_ftp.send_receive_dir import SEND_DIR_DESCRIPTION, SEND_DIR_VALIDATE_DESCRIPTION
#
# from red_connector_ftp.mount_dir import mount_dir, mount_dir_validate, umount_dir
# from red_connector_ftp.mount_dir import MOUNT_DIR_DESCRIPTION, MOUNT_DIR_VALIDATE_DESCRIPTION, UMOUNT_DIR_DESCRIPTION


CLI_VERSION = '1'
SCRIPT_NAME = 'red-connector-ftp'
DESCRIPTION = 'RED Connector FTP'
TITLE = 'modes'

MODES = OrderedDict([
    ('cli-version', {'main': lambda: print(CLI_VERSION), 'description': 'RED connector CLI version.'}),
    ('receive-file', {'main': receive_file, 'description': RECEIVE_FILE_DESCRIPTION}),
    ('receive-file-validate', {'main': receive_file_validate, 'description': RECEIVE_FILE_VALIDATE_DESCRIPTION}),
    # ('send-file', {'main': send_file, 'description': SEND_FILE_DESCRIPTION}),
    # ('send-file-validate', {'main': send_file_validate, 'description': SEND_FILE_VALIDATE_DESCRIPTION}),
    # ('receive-dir', {'main': receive_dir, 'description': RECEIVE_DIR_DESCRIPTION}),
    # ('receive-dir-validate', {'main': receive_dir_validate, 'description': RECEIVE_DIR_VALIDATE_DESCRIPTION}),
    # ('send-dir', {'main': send_dir, 'description': SEND_DIR_DESCRIPTION}),
    # ('send-dir-validate', {'main': send_dir_validate, 'description': SEND_DIR_VALIDATE_DESCRIPTION}),
    # ('mount-dir', {'main': mount_dir, 'description': MOUNT_DIR_DESCRIPTION}),
    # ('mount-dir-validate', {'main': mount_dir_validate, 'description': MOUNT_DIR_VALIDATE_DESCRIPTION}),
    # ('umount-dir', {'main': umount_dir, 'description': UMOUNT_DIR_DESCRIPTION}),
])


def main():
    cli_modes(SCRIPT_NAME, TITLE, DESCRIPTION, MODES, VERSION)



from collections import OrderedDict

from red_connector_ftp.commons.cli_modes import cli_modes
from red_connector_ftp.version import VERSION

from red_connector_ftp.ftp_archive.receive_dir import receive_dir, receive_dir_validate
from red_connector_ftp.ftp_archive.receive_dir import RECEIVE_DIR_DESCRIPTION, RECEIVE_DIR_VALIDATE_DESCRIPTION


CLI_VERSION = '1'
SCRIPT_NAME = 'red-connector-ftp-archive'
DESCRIPTION = 'RED Connector FTP Archive'
TITLE = 'modes'

MODES = OrderedDict([
    ('cli-version', {'main': lambda: print(CLI_VERSION), 'description': 'RED connector CLI version.'}),
    ('receive-dir', {'main': receive_dir, 'description': RECEIVE_DIR_DESCRIPTION}),
    ('receive-dir-validate', {'main': receive_dir_validate, 'description': RECEIVE_DIR_VALIDATE_DESCRIPTION})
])


def main():
    cli_modes(SCRIPT_NAME, TITLE, DESCRIPTION, MODES, VERSION)



from copy import deepcopy

FILE_SCHEMA = {
    'type': 'object',
    'properties': {
        'url': {'type': 'string'},
    },
    'additionalProperties': False,
    'required': ['url']
}

ARCHIVE_SCHEMA = deepcopy(FILE_SCHEMA)
ARCHIVE_SCHEMA['properties']['archiveFormat'] = {'enum': ['zip', 'tar', 'gztar', 'bztar', 'xztar']}
ARCHIVE_SCHEMA['required'].append('archiveFormat')



import os
import sys
from functools import wraps

import jsonschema


def graceful_error(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)

        except jsonschema.exceptions.ValidationError as e:
            if hasattr(e, 'context'):
                print('{}:{}Context: {}'.format(repr(e), os.linesep, e.context), file=sys.stderr)
                exit(1)

            print(repr(e), file=sys.stderr)
            exit(2)

        except Exception as e:
            print('{}: {}'.format(type(e).__name__, e), file=sys.stderr)
            exit(3)

    return wrapper


class InvalidAccessInformationError(Exception):
    pass



import os
import sys
import tempfile
import json
from argparse import ArgumentParser
from ftplib import FTP
from urllib.parse import urlparse

import jsonschema
import shutil

from red_connector_ftp.commons.helpers import InvalidAccessInformationError, graceful_error
from red_connector_ftp.commons.schemas import ARCHIVE_SCHEMA

RECEIVE_DIR_DESCRIPTION = 'Receive input dir from FTP server.'
RECEIVE_DIR_VALIDATE_DESCRIPTION = 'Validate access data for receive-dir.'


def _receive_dir(access, local_dir_path, listing):
    del listing  # ignore listing
    with open(access) as f:
        access = json.load(f)

    url = access.get('url')
    if url is None:
        raise InvalidAccessInformationError('Could not find "url" in access information.')

    parsed_url = urlparse(url)

    with FTP(parsed_url.netloc) as ftp_client:
        ftp_client.login()

        with tempfile.NamedTemporaryFile(suffix='.zip') as archive_output_file:
            ftp_client.retrbinary('RETR {}'.format(parsed_url.path), archive_output_file.write)
            archive_output_file.flush()

            shutil.unpack_archive(archive_output_file.name, local_dir_path, access['archiveFormat'])


def _receive_dir_validate(access, listing):
    del listing  # ignore listing

    with open(access) as f:
        access = json.load(f)

    jsonschema.validate(access, ARCHIVE_SCHEMA)


@graceful_error
def receive_dir():
    parser = ArgumentParser(description=RECEIVE_DIR_DESCRIPTION)
    parser.add_argument(
        'access', action='store', type=str, metavar='ACCESSFILE',
        help='Local path to ACCESSFILE in JSON format.'
    )
    parser.add_argument(
        'local_dir_path', action='store', type=str, metavar='LOCALDIR',
        help='Local input dir path.'
    )
    parser.add_argument(
        '--listing', action='store', type=str, metavar='LISTINGFILE',
        help='Local path to LISTINGFILE in JSON format.'
    )
    args = parser.parse_args()
    _receive_dir(**args.__dict__)


@graceful_error
def receive_dir_validate():
    parser = ArgumentParser(description=RECEIVE_DIR_VALIDATE_DESCRIPTION)
    parser.add_argument(
        'access', action='store', type=str, metavar='ACCESSFILE',
        help='Local path to ACCESSFILE in JSON format.'
    )
    parser.add_argument(
        '--listing', action='store', type=str, metavar='LISTINGFILE',
        help='Local path to LISTINGFILE in JSON format.'
    )
    args = parser.parse_args()
    _receive_dir_validate(**args.__dict__)



import json
import os
from argparse import ArgumentParser
from ftplib import FTP
from urllib.parse import urlparse

import jsonschema
from red_connector_ftp.commons.helpers import graceful_error, InvalidAccessInformationError
from red_connector_ftp.commons.schemas import FILE_SCHEMA

RECEIVE_FILE_DESCRIPTION = 'Receive input file from FTP server.'
RECEIVE_FILE_VALIDATE_DESCRIPTION = 'Validate access data for receive-file.'


def _receive_file(access, local_file_path):
    with open(access) as f:
        access = json.load(f)

    if not os.path.isdir(os.path.dirname(local_file_path)):
        raise NotADirectoryError(
            'Could not create local file "{}". The parent directory does not exist.'.format(local_file_path)
        )

    url = access.get('url')
    if url is None:
        raise InvalidAccessInformationError('Could not find "url" in access information.')

    parsed_url = urlparse(url)

    with FTP(parsed_url.netloc) as ftp_client:
        ftp_client.login()

        with open(local_file_path, 'wb') as output_file:
            ftp_client.retrbinary('RETR {}'.format(parsed_url.path), output_file.write)


def _receive_file_validate(access):
    with open(access) as f:
        access = json.load(f)

    jsonschema.validate(access, FILE_SCHEMA)


# @graceful_error
def receive_file():
    parser = ArgumentParser(description=RECEIVE_FILE_DESCRIPTION)
    parser.add_argument(
        'access', action='store', type=str, metavar='ACCESSFILE',
        help='Local path to ACCESSFILE in JSON format.'
    )
    parser.add_argument(
        'local_file_path', action='store', type=str, metavar='LOCALFILE',
        help='Local output file path.'
    )
    args = parser.parse_args()
    _receive_file(**args.__dict__)


@graceful_error
def receive_file_validate():
    parser = ArgumentParser(description=RECEIVE_FILE_VALIDATE_DESCRIPTION)
    parser.add_argument(
        'access', action='store', type=str, metavar='ACCESSFILE',
        help='Local path to ACCESSFILE in JSON format.'
    )
    args = parser.parse_args()
    _receive_file_validate(**args.__dict__)


